/**
 * @file backend.d
 * @brief Representation of a generated D program
 * @author Sebastien Alaiwan - Romain Bouqueau
 * @date 2014-12-02
 */

import codestream;
import std.string;

class GProgram
{
  GStruct[] structs;
  GFunction[] functions;
}

class GStruct
{
  string name;

  GStructMember[] members;
}

class GStructMember
{
  string type;
  string name;
  int arrayLength;
  string comment;
}

class GFunction
{
  string name;
  string retType;
  string[] args;
  GCodeline[] codelines;
}

class GCodeline
{
public:
  this(GCodelineCallFunction cc)
  {
    m_type = Type.CallFunction;
    m_callFunction = cc;
  }

  this(GCodelineOperation co)
  {
    m_type = Type.Operation;
    m_operation = co;
  }

  void visit(void delegate(const GCodelineCallFunction,
                           CodeStream s) visitCallFunction,
             void delegate(const GCodelineOperation,
                           CodeStream s) visitOperation,
             CodeStream s) const
  {
    final switch(m_type)
    {
    case Type.CallFunction:
      visitCallFunction(m_callFunction, s);
      break;
    case Type.Operation:
      visitOperation(m_operation, s);
      break;
    }
  }

private:
  enum Type
  {
    CallFunction,
    Operation,
  }

  Type m_type;

  union
  {
    GCodelineCallFunction m_callFunction;
    GCodelineOperation m_operation;
  }
}

class GCodelineOperation
{
  string left;
  string operator;
  string right;
}

class GCodelineCallFunction
{
  string codeline;
}

GCodeline mkOperation(string left, string operator, string right)
{
  auto r = new GCodelineOperation;
  r.left = left;
  r.operator = operator;
  r.right = right;
  return new GCodeline(r);
}

GCodeline mkCallFunction(string codeline)
{
  auto r = new GCodelineCallFunction;
  r.codeline = codeline;
  return new GCodeline(r);
}

interface Backend
{
  void printCode(const GProgram p, CodeStream s);

private:
  void printStruct(const GStruct gstruct, CodeStream s);
  void printFunction(const GFunction gfunction, CodeStream s);
  void printCodeline(const GCodeline line, CodeStream s);
}

