import std.stdio;

alias ushort WORD;
alias uint DWORD;
alias int LONG;
alias ubyte BYTE;
alias uint BITS;

class Stream
{
  this(string filename)
  {
    f = File(filename);

    bits = 0;
    remains = 0;
  }

  File f;

  static const uint MAX_BITS_NUM = uint.sizeof * 8;

  uint bits;
  uint remains;
}

BYTE parse_BYTE(Stream s, int iSizeInBits)
{
  assert(iSizeInBits == BYTE.sizeof * 8);
  BYTE c[1];
  s.f.rawRead(c);
  return c[0];
}

WORD parse_WORD(Stream s, int iSizeInBits)
{
  assert(iSizeInBits == WORD.sizeof * 8);
  BYTE c[2];
  s.f.rawRead(c);
  return (c[1] << 8) + c[0];
}

DWORD parse_DWORD(Stream s, int iSizeInBits)
{
  assert(iSizeInBits == DWORD.sizeof * 8);
  BYTE c[4];
  s.f.rawRead(c);
  DWORD r = 0;

  for(int i = 0; i < 4; ++i)
    r += c[i] << (8 * i);

  return r;
}

LONG parse_LONG(Stream s, int iSizeInBits)
{
  assert(iSizeInBits == LONG.sizeof * 8);
  BYTE c[4];
  s.f.rawRead(c);
  DWORD r = 0;

  for(int i = 0; i < 4; ++i)
    r += c[i] << (8 * i);

  return r;
}

/* Note: 1 <= iSizeInBits < Stream.MAX_BITS_NUM */
uint parse_BITS(Stream bitstream, int iSizeInBits)
{
  assert(iSizeInBits <= Stream.MAX_BITS_NUM);
  assert(iSizeInBits >= 1);

  uint ret = 0;

  if(iSizeInBits > bitstream.remains)
  {
    ret = bitstream.bits;
    iSizeInBits -= bitstream.remains;
    ret <<= iSizeInBits;
    bitstream.bits = parse_DWORD(bitstream, 32);
    bitstream.remains = Stream.MAX_BITS_NUM;
  }

  bitstream.remains -= iSizeInBits;
  ret |= bitstream.bits >> bitstream.remains;
  bitstream.bits &= (1 << bitstream.remains) - 1;

  return ret;
}

void print_LONG(string name, LONG val, File o)
{
  o.writefln("%s: %s", name, val);
}

void print_DWORD(string name, DWORD val, File o)
{
  o.writefln("%s: 0x%.8X", name, val);
}

void print_WORD(string name, WORD val, File o)
{
  o.writefln("%s: 0x%.4X", name, val);
}

void print_BYTE(string name, BYTE val, File o)
{
  o.writefln("%s: 0x%.2X", name, val);
}

void print_BITS(string name, BITS val, File o)
{
  o.writefln("%s: 0x%X", name, val);
}

